#!/usr/local/bin/perl
#
#   startproxy.cgi--  Part of CGIProxy 1.1:  Initiate a browsing session
#     through nph-proxy.cgi.
#
#   Copyright (C) 1996, 1998-1999 by James Marshall, james@jmarshall.com
#
#   For the latest, see http://www.jmarshall.com/tools/cgiproxy/
#
#   This script initiates a browsing session through nph-proxy.cgi, prompting
#     the user with a form if needed.  Generates an appropriate "Location:"
#     response with the target URL correctly encoded in PATH_INFO.
#   Efficiency isn't as critical here as in the main nph-proxy.cgi script,
#     since this is normally called once per browsing session.  Therefore,
#     we should offload as much out of nph-proxy.cgi into this script as
#     possible.
#
#
#   CONFIGURATION:
#     None, unless defaults in nph-proxy.cgi are changed:  
#       . If the name of nph-proxy.cgi is changed, set $proxyname below.
#       . Make sure &proxy_encode() matches the one in nph-proxy.cgi.
#

#------ user configuration ------------------------------------------------

# If you change the name of the main proxy script, reflect it here.
$proxyname= 'nph-proxy.cgi' ;


# Create your own proxy_encode() and proxy_decode() to tranform the target
#   URL to and from the format that will be stored in PATH_INFO.  The encoded
#   form should only contain characters that are legal in PATH_INFO, i.e.
#   most printable chars except ? and #, and no two adjacent slashes ("//").
#   Try not to make the resulting string too long, either.
# This must match what is in nph-proxy.cgi, or whatever you called it.

sub proxy_encode {
    local($URL)= @_ ;
    $URL=~ s#^([\w+.-]+)://#$1/# ;                 # http://xxx -> http/xxx
#    $URL=~ s/(.)/ sprintf('%02x',ord($1)) /ge ;   # each char -> 2-hex
#    $URL=~ tr/a-zA-Z/n-za-mN-ZA-M/ ;              # rot-13
    return $URL ;
}

#------ end of (useful) user configuration --------------------------------


$PROXY_VERSION= '1.1' ;

# Read the URL from the query input
($ENV{'QUERY_STRING'}=~ /^URL=([^&]*)/)  || &showstartform ;
$URL= $1 ;

# un-encode the URL
$URL=~ s/\+/ /g ;
$URL=~ s/%([\da-fA-F]{2})/pack("c", hex($1))/ge ;

# Warn the user against FTP or other URL's
($scheme)= $URL=~ /^([\w+.-]+):/ ;
(($scheme eq '') || ($scheme=~ /^http$/i))
    || &HTMLdie("Sorry, only HTTP browsing is currently supported.") ;

# Support abbreviated URL entries (but only HTTP)
$URL=~ s#^http:##i ;
$URL=~ s#^//##i ;
($host, $port, $path)= ($URL=~ m#^([^/?:]*)(:?[^?/]*)(.*)?$#) ;
$host= "www.$host.com" unless $host=~ /\./ ;
$path= "/$path" if $path!~ m#^/# ;    # if path is '' or contains only query

$URL= &proxy_encode("http://$host$port$path") ;

# Print the Location: header
print "Location: $proxyname/$URL\n\n" ;

exit ;



# Present entry form
sub showstartform {
    print <<EOF ;
Content-type: text/html

<html>
<head>
<title>Start Using CGI Proxy</title>
</head>
<body>

<h1>CGI Proxy</h1>

<p>Start browsing through this CGI-based HTTP proxy by entering a URL
below.  Not all functions will work (e.g. HTTP Basic authentication or
some JavaScript), but most pages will be fine.

<form action="$ENV{'SCRIPT_NAME'}" method=get>
<input name="URL" size=50>
<p><input type=submit value="   Begin browsing   ">
</form>
<p>
<hr>
<a href="http://www.jmarshall.com/tools/cgiproxy/"><i>CGIProxy $PROXY_VERSION</i></a>
<p>
</body>
</html>
EOF

    exit ;
}


# Die, outputting HTML error page
sub HTMLdie {
    local($msg)= @_ ;
    print <<EOF ;
Content-type: text/html

<html>
<head>
<title>CGI Proxy Error</title>
</head>
<body>
<h1>CGI
 Proxy Error</h1>
<h3>$msg</h3>
<p>
<hr>
<a href="http://www.jmarshall.com/tools/cgiproxy/"><i>CGIProxy $PROXY_VERSION</i></a>
<p>
</body>
</html>
EOF

    exit ;
}
